#include "CryptoProSignBackend.h"

CryptoProSignatureVerification::CryptoProSignatureVerification(std::vector<unsigned char> const& pkcs7data)
{
    m_pkcs7data = std::move(pkcs7data);
}

SignatureValidationStatus CryptoProSignatureVerification::validateSignature()
{
    // Not implemented
    return SignatureValidationStatus();
}

struct ValidationData {
    std::string& pdfFile;
    std::vector<std::pair<uint64_t, uint64_t>>& byteRanges;
    pdfcsp::poppler::BytesVector& pkcs7data;
    std::unique_ptr<pdfcsp::poppler::ESInfo>& out;
};

SignatureValidationStatus CryptoProSignatureVerification::validateSignature(std::string& pdfFile, std::vector<std::pair<uint64_t, uint64_t>>&& byteRanges)
{
    m_esinfo = std::make_unique<pdfcsp::poppler::ESInfo>(pdfcsp::poppler::CheckES(byteRanges, m_pkcs7data, pdfFile));

    auto& signStatus = m_esinfo->signature_val_status;
    if (static_cast<SignatureValidationStatus>(signStatus) == SignatureValidationStatus::SIGNATURE_VALID) {
        return SignatureValidationStatus::SIGNATURE_VALID;
    }

    return SignatureValidationStatus::SIGNATURE_INVALID;
}

void CryptoProSignatureVerification::addData(unsigned char *dataBlock, int dataLen)
{
    // Not implemented
}

std::chrono::system_clock::time_point CryptoProSignatureVerification::getSigningTime() const
{
    return std::chrono::system_clock::from_time_t(m_esinfo->signing_time);
}

std::string CryptoProSignatureVerification::getSignerName() const
{
    return m_esinfo->signer_name;
}

std::string CryptoProSignatureVerification::getSignerSubjectDN() const
{
    return m_esinfo->signer_subject_dn;
}

HashAlgorithm CryptoProSignatureVerification::getHashAlgorithm() const
{
    return static_cast<HashAlgorithm>(m_esinfo->hash_algorithm);
}

CertificateValidationStatus CryptoProSignatureVerification::validateCertificate(std::chrono::system_clock::time_point validation_time, bool ocspRevocationCheck, bool useAIACertFetch)
{
    // not implemented
    return CertificateValidationStatus();
}

std::string cvtToHexStr(std::string s)
{
    if (s.size() % 2) {
        return s;
    }

    std::string hexStr;
    for (size_t i = 0; i < s.size(); i += 2) {
        std::string sub_str = s.substr(i, 2);
        hexStr += std::stoi(sub_str, nullptr, 16);
    }

    return hexStr;
}

std::unique_ptr<X509CertificateInfo> CryptoProSignatureVerification::getCertificateInfo() const
{
    auto certInfo = std::make_unique<X509CertificateInfo>();

    {
        X509CertificateInfo::EntityInfo issuerInfo({
                m_esinfo->cert_info.issuer_info.commonName,
                m_esinfo->cert_info.issuer_info.distinguishedName,
                m_esinfo->cert_info.issuer_info.email,
                m_esinfo->cert_info.issuer_info.organization});

        certInfo->setIssuerInfo(std::move(issuerInfo));
    }

    {
        X509CertificateInfo::EntityInfo subjectInfo({
                m_esinfo->cert_info.subject_info.commonName,
                m_esinfo->cert_info.subject_info.distinguishedName,
                m_esinfo->cert_info.subject_info.email,
                m_esinfo->cert_info.subject_info.organization});

        certInfo->setSubjectInfo(std::move(subjectInfo));
    }

    {
        X509CertificateInfo::PublicKeyInfo pubKeyInfo;

        std::string publicKey = "";
        for (auto&& elem : m_esinfo->cert_info.public_key_info.publicKey) {
            publicKey += elem;
        }

        pubKeyInfo.publicKey = GooString(std::move(publicKey)),
        pubKeyInfo.publicKeyType = static_cast<PublicKeyType>(m_esinfo->cert_info.public_key_info.publicKeyType);
        pubKeyInfo.publicKeyStrength = m_esinfo->cert_info.public_key_info.publicKeyStrength;

        certInfo->setPublicKeyInfo(std::move(pubKeyInfo));
    }

    {
        X509CertificateInfo::Validity validity;

        validity.notBefore = m_esinfo->cert_info.cert_validity.notBefore;
        validity.notAfter = m_esinfo->cert_info.cert_validity.notAfter;

        certInfo->setValidity(validity);
    }

    certInfo->setSerialNumber(GooString(cvtToHexStr(m_esinfo->cert_info.cert_serial)));

    std::string certDer = "";
    for (auto&& elem : m_esinfo->cert_info.cert_der) {
        certDer += elem;
    }


    certInfo->setCertificateDER(GooString(std::move(certDer)));
    certInfo->setNickName(GooString(std::move(m_esinfo->cert_info.cert_nick)));
    certInfo->setKeyUsageExtensions(m_esinfo->cert_info.ku_extensions);
    certInfo->setVersion(m_esinfo->cert_info.cert_version);
    certInfo->setIsSelfSigned(m_esinfo->cert_info.is_self_signed);
    certInfo->setKeyLocation(static_cast<KeyLocation>(m_esinfo->cert_info.keyLocation));

    return certInfo;
}

std::unique_ptr<CryptoSign::VerificationInterface> CryptoProSignBackend::createVerificationHandler(std::vector<unsigned char> &&pkcs7)
{
    return std::make_unique<CryptoProSignatureVerification>(std::move(pkcs7));
}

std::unique_ptr<CryptoSign::SigningInterface> CryptoProSignBackend::createSigningHandler(std::string const& certID, HashAlgorithm digestAlgTag)
{
    // Not implemented

    return std::unique_ptr<CryptoSign::SigningInterface>();
}

std::vector<std::unique_ptr<X509CertificateInfo>> CryptoProSignBackend::getAvailableSigningCertificates()
{
    // Not implemented

    return std::vector<std::unique_ptr<X509CertificateInfo>>();
}

CryptoProSignBackend::~CryptoProSignBackend() = default;
